/*
 * Copyright © 2021-2023 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of ipspiel24-GD.
 *
 * ipspiel24-GD is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * ipspiel24-GD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with ipspiel24-GD. If not, see
 * <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.GefangenenDilemma.strategy.stratsWithBrain;

import java.util.List;
import java.util.Map.Entry;
import java.util.Optional;

import de.fhdw.gaming.GefangenenDilemma.domain.GDPlayer;
import de.fhdw.gaming.GefangenenDilemma.domain.GDState;
import de.fhdw.gaming.GefangenenDilemma.domain.GDStrategy;
import de.fhdw.gaming.GefangenenDilemma.moves.GDMove;
import de.fhdw.gaming.GefangenenDilemma.moves.factory.GDMoveFactory;
import de.fhdw.gaming.GefangenenDilemma.moves.impl.GDSnitchMove;
import de.fhdw.gaming.core.domain.Move;
import de.fhdw.gaming.memory.Key;
import de.fhdw.gaming.memory.impl.MemoryBrainImpl;
import de.fhdw.gaming.memory.impl.MemoryKeyImpl;
import de.fhdw.gaming.memory.impl.MemoryPairImpl;

/**
 * Implements {@link GDStrategy} using the Grim Trigger strategy.
 * Starts by cooperating and continues to cooperate unless the opponent defects.
 * One defection leads to permanent defection.
 */
public final class GDGrimTriggerStrategy implements GDStrategy {

    /**
     * The factory for creating Gefangenen-Dilemma moves.
     */
    private final GDMoveFactory moveFactory;

    /**
     * Creates an {@link GDGrimTriggerStrategy}.
     *
     * @param moveFactory The factory for creating Gefangenen-Dilemma moves.
     */
    GDGrimTriggerStrategy(final GDMoveFactory moveFactory) {
        this.moveFactory = moveFactory;
    }

    @Override
    public Optional<GDMove> computeNextMove(
            final int gameId,
            final GDPlayer player,
            final GDState state,
            final long maxComputationTimePerMove) {

        GDPlayer opponent = null;
        for (Entry<String, GDPlayer> entry : state.getPlayers().entrySet()) {
            if (!entry.getValue().equals(player)) {
                opponent = entry.getValue();
            }
        }

        final Key key = new MemoryKeyImpl(player, opponent, this);

        List<MemoryPairImpl> check = null;
        if (MemoryBrainImpl.getInstance().getMemory(key) != null) {
            check = MemoryBrainImpl.getInstance().getMemory(key).show();
        }

        if (check == null || check.isEmpty()) {
            return Optional.of(this.moveFactory.createRemainSilentMove());
        }

        for (MemoryPairImpl memoryPair : check) {
            final Move<?, ?> lastMoveUsed = memoryPair.getOpponentMove();
            if (lastMoveUsed.getClass().equals(GDSnitchMove.class)) {
                return Optional.of(this.moveFactory.createSnitchMove());
            }
        }

        return Optional.of(this.moveFactory.createRemainSilentMove());
    }

    @Override
    public String toString() {
        return GDGrimTriggerStrategy.class.getSimpleName();
    }
}
